﻿using AutoMapper;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Hosting;
using Ys.Scada.Backend.Models;
using Ys.Scada.Core;
using Ys.Scada.Core.EventBus.Publisher;
using Ys.Scada.Core.EventModels;
using Ys.Scada.Repository.Domain;
using Newtonsoft.Json;
using Ubiety.Dns.Core;
using Ys.Scada.Core.Common;
using System.Text;
using System.Collections.Generic;
using Ys.Scada.Manager;
using Ys.Scada.Repository;
using Ys.Scada.RabbitMQ;

namespace Ys.Scada.Backend.Controllers
{
    public class DataSourceController : BaseController
    {
        private readonly IEventPublisher _eventPublisher;
        private readonly IHttpClientFactory _httpClientFactory;
        private readonly DataSourceManager _manager;
        public DataSourceController(IFreeSql fsql, IMapper mapper, IHttpClientFactory httpClientFactory, IEventPublisher eventPublisher, DataSourceManager dataSourceManager) : base(fsql, mapper)
        {
            _eventPublisher = eventPublisher;
            _httpClientFactory = httpClientFactory;
            _manager = dataSourceManager;
        }
        [HttpGet("list")]
        public async Task<JsonResult> List([FromQuery] int page, [FromQuery] int rows, [FromQuery] int docId, [FromQuery] string? filter)
        {
            var data = await _fsql.Select<DataSource>()
                .Where(o => o.DocId == docId)
                .WhereIf(!string.IsNullOrEmpty(filter), o => o.Name.Contains(filter) || o.Type.Contains(filter))
                .OrderByDescending(o => o.CreatedAt)
                .Count(out long count)
                .Page(page, rows)
                .ToListAsync();

            var modes = _mapper.Map<List<DataSourceModel>>(data);
            return OK(new PagedData<DataSourceModel>(modes, page, rows, (int)count));
        }
        /// <summary>
        /// 数据源保存或者编辑，参数信息参看模型说明
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        [HttpPost("save")]
        public async Task<JsonResult> Save(DataSourceModel model)
        {
            var document = await _fsql.Select<Document>().Where(o => o.Id == model.DocId).ToOneAsync();
            Check.NotNull(document);
            if (document.Type?.ToLower() != "project")
                throw new CustomException("非项目，数据错误");
            if (model.Id > 0)
            {
                var entity = _mapper.Map<DataSource>(model);
                await _fsql.Update<DataSource>()
                    .SetSource(entity)
                    .ExecuteAffrowsAsync();
                return OK(model);
            }
            else
            {
                var entity = _mapper.Map<DataSource>(model);
                var id = (int)(await _fsql.Insert(entity).ExecuteIdentityAsync());
                model.Id = id;
                return OK(model);
            }
        }
        [HttpPost("delete")]
        public async Task<JsonResult> Delete(int docId, int id)
        {
            _fsql.Transaction(() =>
            {
                _fsql.Delete<DataSource>()
                   .Where(o => o.Id == id && o.DocId == docId)
                   .ExecuteAffrows();
                _fsql.Delete<Variable>().Where(o => o.SourceId == id && o.DocId == docId).ExecuteAffrows();
            });
            return OK();
        }
        public class ForeignDataPublishOutPut
        {
            public string Id { get; set; }
            public string Name { get; set; }
            public string ProtocolName { get; set; }
        }
        public record ReadVarible(string code);
        public record WriteVarible(string code, string value);
        public record WriteValue(string name, string value);
        [HttpPost("request")]
        public async Task<JsonResult> Request(DataSourceModel source)
        {
            //var source = await _fsql.Select<DataSource>().Where(o => o.Id == id).ToOneAsync();
            //if (source == null)
            //    throw new CustomException("数据不存在");
            Check.NotNull(source);
            Check.NullEmpty(source.Host);
            Check.NullEmpty(source.Port.ToString());
            var host = $"{source.Host}:{source.Port}";
            var result = new List<ForeignDataPublishOutPut>();
            //if (source.Router.ToLower() == "getvariable")
            //{
            //    var data = (source.Data.ToString().Split(',').ToList());
            //    var variableValue = await GetVariable(host, data);
            //    var variableV = JsonConvert.DeserializeObject<ApiResult<List<PointData>>>(variableValue);
            //    return OK(variableV?.Data);
            //}
            //else if (source.Router.ToLower() == "writevariable")
            //{
            //    var data = JsonConvert.DeserializeObject<WriteVarible>(source.Data.ToString());
            //    await WriteVariable(host, data.code, data.value);
            //    return OK();
            //}
            //else
            if (source.Router.ToLower() == "tempwritevariable")
            {
                var writeValue = JsonConvert.DeserializeObject<WriteValue>(source.Data.ToString());
                _manager.Write(writeValue.name, writeValue.value);
            }
            else if (source.Router.ToLower() == "getvariableinfo")
            {
                return OK(_manager.GetTags());
            }
            //var entity = _mapper.Map<DataSource>(source);
            //entity.Ext = JsonConvert.SerializeObject(result);
            //if (entity.Id > 0)
            //{
            //    await _fsql.Update<DataSource>().SetSource(entity)
            //        .IgnoreColumns(o => new { o.Id })
            //        .ExecuteAffrowsAsync();
            //}
            //else
            //{
            //    source.Id = (int)(await _fsql.Insert(entity).ExecuteIdentityAsync());
            //}
            return OK(result);
        }
        [HttpGet("reciver")]
        public async Task<JsonResult> Reciver()
        {
            return OK(_manager.GetTagsData());
        }
        /// <summary>
        /// 测试数据源是否能联通（不拉取实时数据），返回非200表示连接错误
        /// 这里有个逻辑问题，是先保存才能测试还是先测试再保存
        /// 如果是先测试再保存，那么就需要前端给我传数据源连接信息，
        /// 如果是先保存再测试，就只需要给我数据源id即可
        /// </summary>
        /// <param name="id">数据源id</param>
        /// <returns></returns>
        /// <exception cref="CustomException"></exception>
        [HttpPost("testConnect")]
        public async Task<JsonResult> TestConnect(int id)
        {
            var dataSource = await _fsql.Select<DataSource>().Where(o => o.Id == id).ToOneAsync();
            if (dataSource == null)
                throw new CustomException("数据不存在");

            await _manager.TestConnect(dataSource);
            return OK();
        }
        /// <summary>
        /// 启动数据源拉取实时数据
        /// </summary>
        /// <param name="id">数据源id</param>
        /// <returns></returns>
        /// <exception cref="CustomException"></exception>
        [HttpPost("start")]
        public async Task<JsonResult> Start(int id, int docId)
        {
            var document = await _fsql.Select<Document>().Where(o => o.Id == docId).ToOneAsync();
            Check.NotNull(document);
            if (document.Type?.ToLower() != "project")
                throw new CustomException("非法操作");

            var dataSource = await _fsql.Select<DataSource>().Where(o => o.Id == id && o.DocId == docId).ToOneAsync();
            if (dataSource == null)
                throw new CustomException("数据不存在");
            if (dataSource.Status == 1)
                return OK();

            await _manager.TestConnect(dataSource);
            await _manager.Create(dataSource);
            await _fsql.Update<DataSource>(dataSource.Id)
                .Set(o => o.Status, 1)
                .ExecuteAffrowsAsync();
            return OK();
        }
        [HttpPost("stop")]
        public async Task<JsonResult> Stop(int id)
        {
            var dataSource = await _fsql.Select<DataSource>().Where(o => o.Id == id).ToOneAsync();
            Check.NotNull(dataSource);
            await _manager.Cancel(dataSource.Id.ToString());
            return OK();
        }
    }
}
