﻿using AutoMapper;
using Microsoft.AspNetCore.Mvc;
using System.Text.RegularExpressions;
using Ys.Scada.Backend.Models;
using Ys.Scada.Core.Common;
using Ys.Scada.Core;
using Ys.Scada.Repository.Domain;
using System.Reflection.Metadata;
using Document = Ys.Scada.Repository.Domain.Document;
using System.Threading.Channels;

namespace Ys.Scada.Backend.Controllers
{
    public class DocumentController : BaseController
    {
        public DocumentController(IFreeSql fsql, IMapper mapper) : base(fsql, mapper)
        {
        }
        [HttpPost("save")]
        public async Task<JsonResult> Save(DocumentModel documentModel)
        {
            Check.NotNull(documentModel);
            documentModel.UserId = UserId;
            if (documentModel.Type .ToLower() == "project")
                documentModel.Variables = null;

            if (documentModel.Id > 0)
            {
                var oldModel = (await _fsql.Select<Document>().Where(o => o.Id == documentModel.Id).FirstAsync());
                if (oldModel == null)
                {
                    throw new CustomException("数据不存在");
                }
                var model = _mapper.Map<Document>(documentModel);
                await _fsql.Update<Document>().SetSource(model).ExecuteAffrowsAsync();
            }
            else
            {
                var model = _mapper.Map<Document>(documentModel);
                model.CreatedBy = UserId;
                documentModel.Id = (int)(await _fsql.Insert<Document>(model).ExecuteIdentityAsync());
            }
            return OK(documentModel);
        }
        [HttpGet("list")]
        public async Task<JsonResult> List([FromQuery] int page, [FromQuery] int rows, [FromQuery] string? filter, string? type, string? channel)
        {
            if (string.IsNullOrWhiteSpace(channel))
                return OK(new PagedData<DocumentModel>(new List<DocumentModel>(), page, rows, 0));
            var query = _fsql.Select<Document>()
                .WhereIf(!string.IsNullOrEmpty(filter), o => o.Title.Contains(filter) || o.Keywords.Contains(filter));
            query = query.WhereIf(!string.IsNullOrWhiteSpace(type), o => o.Type.Contains(type));
            query = query.Where(o => o.Channel == channel);
            var data = await query.OrderByDescending(o => o.CreatedAt)
               .Count(out long count)
               .Page(page, rows)
               .ToListAsync();
            return OK(new PagedData<DocumentModel>(_mapper.Map<List<DocumentModel>>(data), page, rows, (int)count));
        }
        [HttpGet("myList")]
        public async Task<JsonResult> MyList([FromQuery] int page, [FromQuery] int rows, [FromQuery] string? filter, string? type, string? channel)
        {
            var query = _fsql.Select<Document>()
                .WhereIf(!string.IsNullOrEmpty(filter), o => o.Title.Contains(filter) || o.Keywords.Contains(filter));
            query = query.WhereIf(!string.IsNullOrWhiteSpace(type), o => o.Type.Contains(type));
            query = query.Where(o => o.Channel == channel);
            query = query.Where(o => o.UserId == UserId);
            var data = await query.OrderByDescending(o => o.CreatedAt)
               .Count(out long count)
               .Page(page, rows)
               .ToListAsync();
            return OK(new PagedData<DocumentModel>(_mapper.Map<List<DocumentModel>>(data), page, rows, (int)count));
        }
        [HttpGet("delete")]
        public async Task<JsonResult> Delete(int id)
        {
            var resources = await _fsql.Select<Document>().Where(x => x.Id == id).ToOneAsync();
            Check.NotNull(resources);
            _fsql.Delete<Document>().Where(o => o.Id == id).ExecuteAffrows();
            return OK();
        }
        [HttpGet("get")]
        public async Task<JsonResult> Get(int id)
        {
            var model = (await _fsql.Select<Document>().Where(o => o.Id == id).FirstAsync());
            if (model == null)
                throw new CustomException("数据不存在");
            var viewModel = _mapper.Map<DocumentModel>(model);
            return OK(viewModel);
        }
        [HttpPost("release")]
        public async Task<JsonResult> Release(DocumentReleaseModel documentReleaseModel)
        {
            var oldModel = (await _fsql.Select<Document>().Where(o => o.Id == documentReleaseModel.Id).FirstAsync());
            if (oldModel == null)
            {
                throw new CustomException("数据不存在");
            }
            var model = _mapper.Map<Document>(documentReleaseModel);
            await _fsql.Update<Document>().Set(o => o.Channel, model.Channel).Set(o => o.Description, model.Description).Set(o => o.Title, model.Title).Set(o => o.Keywords, model.Keywords).Where(o=>o.Id == documentReleaseModel.Id).ExecuteAffrowsAsync();
            return OK();
        }
        public record SubscribeValue (string code, string value);
        /// <summary>
        /// 更新实时数据订阅信息
        /// </summary>
        /// <param name="docId"></param>
        /// <returns></returns>
        /// <exception cref="CustomException"></exception>
        [HttpPost("subscribe")]
        public async Task<JsonResult> Subscribe(int docId) 
        {
            var document = await _fsql.Select<Document>().Where(o => o.Id == docId).ToOneAsync();
            Check.NotNull(document);
            if (document.Type?.ToLower() != "page")
                throw new CustomException("项目数据错误");

            var variables = document.Variables ?? "";
            var key = Global.GetUserKey(UserId.ToString());
            ForgeCache.Instance.SetValue(key, variables);

            var result = new List<SubscribeValue>();
            variables.Split(',').ToList().ForEach(o => 
            {
                var variableKey = Global.GetVariableKey(o);
                var value = ForgeCache.Instance.GetValue<string>(variableKey);
                result.Add(new SubscribeValue(o, value));
            });

            return OK(result);
        }
    }
}
