﻿using AutoMapper;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using Microsoft.IdentityModel.Tokens;
using MySqlX.XDevAPI.Relational;
using Org.BouncyCastle.Asn1;
using System.Globalization;
using System.IdentityModel.Tokens.Jwt;
using System.Security.Claims;
using System.Text;
using Ys.Scada.Backend.Authorization;
using Ys.Scada.Backend.Models;
using Ys.Scada.Core;
using Ys.Scada.Core.Common;
using Ys.Scada.Repository;
using Ys.Scada.Service;
using static Ys.Scada.Backend.Controllers.UserController;

namespace Ys.Scada.Backend.Controllers
{
    public class UserController : BaseController
    {
        private readonly ILogger<UserController> _logger;
        private readonly IOptions<JwtSetting> _jwtSetting;
        private readonly IValidateCodeOperate _validateCodeOperate;
        public UserController(ILogger<UserController> logger, IFreeSql fsql, IMapper mapper, IOptions<JwtSetting> jwtSetting, IValidateCodeOperate validateCodeOperate) : base(fsql, mapper)
        {
            _logger = logger;
            _jwtSetting = jwtSetting;
            _validateCodeOperate = validateCodeOperate;
        }
        private string GenerateToken(List<Claim> claims)
        {
            var secretKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(_jwtSetting.Value.SecretKey));

            var creds = new SigningCredentials(secretKey, SecurityAlgorithms.HmacSha256);
            var token = new JwtSecurityToken(
                           null,
                           null,
                           claims,
                           null,
                           null,
                           creds);
            return new JwtSecurityTokenHandler().WriteToken(token);
        }
        public record LoginParams(string Username, string Password, string code,string clientId);
        [AllowAnonymous]
        [HttpPost("login")]
        public async Task<JsonResult> Login(LoginParams loginParams)
        {
            Check.NotNull(loginParams);
            Check.NullEmpty(loginParams.code);
            var code =  ForgeCache.Instance.GetValue<string>(loginParams.clientId);
            if (code == null || code != loginParams.code.ToLower()) { throw new CustomException("验证码错误"); }
            ForgeCache.Instance.DeleteValue(loginParams.clientId);

            var user = await _fsql.Select<User>().Where(x => x.Username == loginParams.Username && x.Password == loginParams.Password).ToOneAsync();
            if (user == null)
                throw new CustomException("账号密码错误");
            if (user.Status != 0)
                throw new CustomException("您的账户已禁用");
            var claims = new List<Claim>
            {
                new Claim(ClaimTypes.NameIdentifier, user.Id.ToString()),
                new Claim(ClaimTypes.Name, user.Username),
                new Claim(System.IdentityModel.Tokens.Jwt.JwtRegisteredClaimNames.Jti, user.Jti.ToString())
            };
            return OK(GenerateToken(claims));
        }
        [HttpGet("list")]
        public async Task<JsonResult> List([FromQuery] int page, [FromQuery] int rows, string? filter )
        {
            var data = await _fsql.Select<User>()
                .WhereIf(!string.IsNullOrEmpty(filter), o => o.Name.Contains(filter) || o.Username.Contains(filter) || o.Phone.Contains(filter))
                .OrderByDescending(o => o.CreatedAt)
                .Count(out long count)
                .Page(page, rows)
                .ToListAsync();

            var modes = _mapper.Map<List<UserModel>>(data);
            return OK(new PagedData<UserModel>(modes, page, rows, (int)count));
        }
        public record UpdatePasswordParam(string oldPassword, string newPassword);
        [HttpPost("password")]
        public async Task<JsonResult> UpdatePassword([FromBody] UpdatePasswordParam password) 
        {
            var id = UserId;
            var model = await _fsql.Select<User>().Where(o=>o.Id == id).FirstAsync();
            if (model == null || model.Password != password.oldPassword)
            {
                throw new CustomException("数据错误");
            }
            var jti = model.Jti + 1;
            await _fsql.Update<User>()
                 .Set(o => o.Password, password.newPassword)
                 .Set(o => o.Jti, jti)
                 .Where(o=>o.Id == id)
                 .ExecuteAffrowsAsync();
            return OK();
        }
        [HttpDelete("{userId}")]
        public async Task<JsonResult> Delete(int userId)
        {
            var user = await _fsql.Select<User>().Where(x => x.Id == userId).ToOneAsync();
            Check.NotNull(user);
            _fsql.Transaction(() =>
            {
                _fsql.Delete<User>(userId).ExecuteAffrows();
                _fsql.Delete<UserRoleMapping>(userId).ExecuteAffrows();
            });
            return OK();
        }
        [AllowAnonymous]
        [HttpPost("save")]
        public async Task<JsonResult> Save(UserExtensModel userModel)
        {
            Check.NotNull(userModel);
            if (userModel.Id > 0)
            {
                _fsql.Transaction(() =>
                {
                    _fsql.Update<User>(userModel.Id)
                   .Set(o => o.Name, userModel.Name)
                   .Set(o => o.Status, userModel.Status)
                   .Set(o => o.IsSystemRole, userModel.IsSystemRole)
                   .Set(o => o.Phone, userModel.Phone)
                   .ExecuteAffrows();

                    _fsql.Delete<UserRoleMapping>().Where(o => o.UserId == userModel.Id).ExecuteAffrows();
                    if (userModel.Roles != null && userModel.Roles.Any())
                    {
                        userModel.Roles.ForEach( o =>
                        {
                            _fsql.Insert<UserRoleMapping>(new UserRoleMapping
                            {
                                UserId = userModel.Id,
                                UserRoleId = o.Id
                            })
                           .ExecuteAffrows();
                        });
                    }
                });
            }
            else
            {
                if( await _fsql.Select<User>().ToOneAsync(o=>o.Username == userModel.UserName))
                    throw new CustomException("用户名已存在");
                _fsql.Transaction(() =>
                {
                    var id = _fsql.Insert<User>(new User
                    {
                        CreatedBy = 0,
                        Name = userModel.Name,
                        IsSystemRole = userModel.IsSystemRole,
                        Jti = 1,
                        Password = userModel.Password,
                        Username = userModel.UserName,
                        Phone = userModel.Phone,
                        Status = userModel.Status
                    }).ExecuteIdentity();

                    if (userModel.Roles != null && userModel.Roles.Any())
                    {
                        userModel.Roles.ForEach(async o =>
                        {
                            _fsql.Insert<UserRoleMapping>(new UserRoleMapping
                            {
                                UserId = userModel.Id,
                                UserRoleId = o.Id
                            })
                           .ExecuteAffrows();
                        });
                    }
                    userModel.Id = (int)id;
                });
            }
            return OK(userModel);
        }
        [HttpGet]
        public async Task<JsonResult> Get()
        {
            var user = await _fsql.Select<User>().IncludeMany(o => o.Mapping).Where(o => o.Id == UserId).ToOneAsync();
            if (user == null)
                return null;
            var roleIds = user.Mapping.Select(o => o.UserRoleId);
            var roles = await _fsql.Select<UserRole>().Where(o => roleIds.Contains(o.Id)).ToListAsync();

            var permissionMapping = await _fsql.Select<PermissionUserRoleMapping>().Where(o => roleIds.Contains(o.RoleId)).ToListAsync();
            var permissionIds = permissionMapping.Select(o => o.PermissionId);
            var permissions = await _fsql.Select<Permission>().Where(o => permissionIds.Contains(o.Id)).ToListAsync();

            return OK(new UserExtensModel
            {
                Id = UserId,
                IsSystemRole = user.IsSystemRole,
                LastLoginTime = user.LastLoginTime,
                Name = user.Name,
                Permissions = _mapper.Map<List<PermissionModel>>(permissions),
                Roles = _mapper.Map<List<RoleModel>>(roles),
                Status = user.Status,
                UserName = user.Username
            });
        }
        public record CodeRecord(string clientId, string code);
        [HttpGet("code")]
        [AllowAnonymous]
        public async Task<JsonResult> GetValidateCode() 
        {
            var (code, image) = await _validateCodeOperate.CreateValidate(4, 120, 32, 24);
            var clientId = Guid.NewGuid().ToString("N");
            ForgeCache.Instance.SetValue(clientId, code.ToLower());
            return OK(new CodeRecord(clientId, image));
        }
    }
}
