﻿using AutoMapper;
using Microsoft.AspNetCore.Mvc;
using System.Runtime.InteropServices;
using Ys.Scada.Backend.Models;
using Ys.Scada.Core;
using Ys.Scada.Repository.Domain;

namespace Ys.Scada.Backend.Controllers
{
    public class VariableController : BaseController
    {
        private List<VariableModel> variables = new List<VariableModel>()
        {
            new VariableModel
            {
                 AliasCode = "code",
                   Code = "code",
                    DataType = "string",
                     Description = "description",
                       Name = "name",
                        ReadWrite = "读",
                         Unit = "cm",
                          VariableAddress = "address",
            }
        };
        public VariableController(IFreeSql fsql, IMapper mapper) : base(fsql, mapper)
        {
        }
        /// <summary>
        /// 删除三方数据源标签
        /// </summary>
        /// <param name="docId">项目id（非页面id）</param>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpPost("delete")]
        public async Task<JsonResult> DeleteVariable(int docId, int id)
        {
            _fsql.Delete<Variable>()
               .Where(o => o.Id == id && o.DocId == docId)
               .ExecuteAffrows();
            return OK();
        }
        /// <summary>
        /// 三方数据源标签列表
        /// </summary>
        /// <param name="page"></param>
        /// <param name="rows"></param>
        /// <param name="filter"></param>
        /// <param name="docId">项目id（非页面id）</param>
        /// <returns></returns>
        [HttpPost("list")]
        public async Task<JsonResult> ListVariable([FromQuery] int page, [FromQuery] int rows, [FromQuery] string? filter, [FromQuery] int docId)
        {
            var query = _fsql.Select<Variable>().Where(o => o.DocId == docId);
            query = query.WhereIf(!string.IsNullOrWhiteSpace(filter), o => o.Code.Contains(filter) || o.Name.Contains(filter) || o.ReadWrite.Contains(filter) || o.DataType.Contains(filter) || o.AliasCode.Contains(filter) || o.Unit.Contains(filter));
            var data = await query.OrderByDescending(o => o.CreatedAt)
               .Count(out long count)
               .Page(page, rows)
               .ToListAsync();
            return OK(new PagedData<VariableModel>(_mapper.Map<List<VariableModel>>(data), page, rows, (int)count));
        }
        /// <summary>
        /// 拉取三方数据源配置信息有三种方式，导入、手动创建、通道对接。该接口为手动创建标签
        /// </summary>
        /// <param name="docId">项目id（非页面id）</param>
        /// <param name="variables">标签集合</param>
        /// <returns></returns>
        /// <exception cref="CustomException"></exception>
        [HttpPost("save")]
        public async Task<JsonResult> CreateVariable(int docId, int sourceId, List<VariableModel> variables)
        {
            var document = await _fsql.Select<Document>().Where(o => o.Id == docId).ToOneAsync();
            Check.NotNull(document);
            if (document.Type?.ToLower() != "project")
                throw new CustomException("项目数据错误");

            var newEntitys = _mapper.Map<List<Variable>>(variables);
            var oldEntitys = await _fsql.Select<Variable>().Where(o => o.DocId == docId && o.SourceId == sourceId).ToListAsync();
            var entitys = new List<Variable>();
            var removes = new List<Variable>();
            foreach (var v in newEntitys)
            {
                if (string.IsNullOrWhiteSpace(v.Code) || string.IsNullOrWhiteSpace(v.VariableAddress) || string.IsNullOrWhiteSpace(v.ReadWrite))
                    continue;
                var variable = oldEntitys.Where(o => o.Code == v.Code).FirstOrDefault();
                if (variable != null)
                    removes.Add(variable);
                entitys.Add(v);
            }
            _fsql.Transaction(() =>
            {
                if (removes.Any())
                    _fsql.Delete<Variable>().Where(removes).ExecuteAffrows();
                _fsql.Insert(entitys).ExecuteAffrows();
            });
            return OK();
        }
        /// <summary>
        /// 导入三方数据源配置信息
        /// </summary>
        /// <param name="docId">项目id（非页面id）</param>
        /// <param name="formFile">excel文件</param>
        /// <returns></returns>
        /// <exception cref="CustomException"></exception>
        [HttpPost("import")]
        public async Task<JsonResult> ImportVarible(int docId, int sourceId, IFormFile formFile)
        {
            var document = await _fsql.Select<Document>().Where(o => o.Id == docId).ToOneAsync();
            Check.NotNull(document);
            if (document.Type?.ToLower() != "project")
                throw new CustomException("项目数据错误");

            if (base.Request.Form.Files.Count != 1)
                throw new CustomException("请上传文件");
            var file = base.Request.Form.Files[0];
            string ext = Path.GetExtension(file.FileName).ToLower();
            if (ext != (".xlsx") && ext != ".xls" && ext != ".csv")
                throw new CustomException("文件不合法");

            var variables = file.OpenReadStream().ExcelToEntity<VariableModel>();
            var varis = variables.Select(o =>
            {
                o.DocId = docId;
                o.SourceId = sourceId;
                return o;
            });
            var newEntitys = _mapper.Map<List<Variable>>(varis);
            var oldEntitys = await _fsql.Select<Variable>().Where(o => o.DocId == docId && o.SourceId == sourceId).ToListAsync();
            var entitys = new List<Variable>();
            foreach (var v in newEntitys)
            {
                if (string.IsNullOrWhiteSpace(v.Code) || string.IsNullOrWhiteSpace(v.VariableAddress) || string.IsNullOrWhiteSpace(v.ReadWrite))
                    continue;
                var variable = oldEntitys.Where(o => o.Code == v.Code).FirstOrDefault();
                if (variable != null)
                    oldEntitys.Remove(variable);
                entitys.Add(v);
            }
            entitys = newEntitys.Union(oldEntitys).ToList();
            _fsql.Transaction(() =>
            {
                _fsql.Delete<Variable>().Where(o => o.DocId == docId && o.SourceId == sourceId).ExecuteAffrows();
                _fsql.Insert(entitys).ExecuteAffrows();
            });
            return OK();
        }
        /// <summary>
        /// 下载导入模板
        /// </summary>
        /// <returns></returns>
        [HttpPost("template")]
        public async Task<FileContentResult> DownloadVariable()
        {
            var byt = variables.ExportToExcelByte("variable");
            return new FileContentResult(byt, "application/vnd.ms-excel")
            {
                FileDownloadName = $"{Guid.NewGuid()}.xlsx"
            };
        }
    }
}
