﻿using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.AspNetCore.Mvc.ModelBinding;
using Microsoft.AspNetCore.Mvc.ModelBinding.Binders;
using Microsoft.Extensions.FileProviders;
using Microsoft.IdentityModel.Tokens;
using Microsoft.OpenApi.Models;
using Newtonsoft.Json;
using System.Text;
using System.Text.Json.Serialization;
using System.Threading.Channels;
using Ys.Scada.Backend.Authorization;
using Ys.Scada.Backend.EventConsumers;
using Ys.Scada.Backend.WS;
using Ys.Scada.Core;
using Ys.Scada.Core.EventBus.Consumer;
using Ys.Scada.Core.EventBus.Publisher;
using Ys.Scada.Core.EventBus.Subscriber;
using Ys.Scada.Core.MQ;
using Ys.Scada.Manager;

namespace Ys.Scada.Backend
{
    public class Startup
    {
        public IConfiguration Configuration { get; }
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }
        public void ConfigureServices(IServiceCollection services) 
        {
            services.AddControllers(options =>
            {
                options.ModelBinderProviders.RemoveType<DateTimeModelBinderProvider>();
            }).AddNewtonsoftJson(options =>
            {
                options.SerializerSettings.DateFormatString = "yyyy-MM-dd HH:mm:ss";
                JsonConvert.DefaultSettings = () => options.SerializerSettings;
            });
            services.AddSwaggerGen(c =>
            {
                c.SwaggerDoc("v1", new OpenApiInfo { Title = "DeepBerry" });
                var securitySchema = new OpenApiSecurityScheme
                {
                    Description = "JWT Authorization header using the Bearer scheme. Example: \"Authorization: Bearer {token}\"",
                    Name = "Authorization",
                    In = ParameterLocation.Header,
                    Type = SecuritySchemeType.Http,
                    Scheme = "Bearer",
                    Reference = new OpenApiReference
                    {
                        Type = ReferenceType.SecurityScheme,
                        Id = "Bearer"
                    }
                };
                c.AddSecurityDefinition("Bearer", securitySchema);
                var securityRequirement = new OpenApiSecurityRequirement
                {
                    { securitySchema, Array.Empty<string>() }
                };
                c.AddSecurityRequirement(securityRequirement);
                c.IncludeXmlComments("Ys.Scada.Backend.xml");
            });
            services.AddSwaggerGenNewtonsoftSupport();
            
            services.Configure<JwtSetting>(Configuration.GetSection("JwtSetting"));
            JwtSetting setting = new JwtSetting();
            Configuration.Bind("JwtSetting", setting);
            services.AddAuthentication(option =>
            {
                option.DefaultAuthenticateScheme = JwtBearerDefaults.AuthenticationScheme;
                option.DefaultChallengeScheme = JwtBearerDefaults.AuthenticationScheme;
            }).AddJwtBearer(JwtBearerDefaults.AuthenticationScheme, config =>
            {
                config.TokenValidationParameters = new TokenValidationParameters
                {
                    ValidateIssuer = false,
                    ValidateAudience = false,
                    ValidateLifetime = true, 
                    RequireExpirationTime = false,
                    IssuerSigningKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(setting.SecretKey)),
                    ClockSkew = TimeSpan.FromSeconds(5) 
                };
                config.Events = new JwtBearerEvents()
                {
                    OnMessageReceived = context =>
                    {
                        var accessToken = context.Request.Query["access_token"];
                        if (!string.IsNullOrEmpty(accessToken))
                        {
                            context.Token = accessToken;
                        }
                        return Task.CompletedTask;
                    },
                };
            });
            services.AddCors(options =>
            {
                options.AddPolicy(name: "AllowAll",
                    builder =>
                    {
                        builder.AllowAnyOrigin() 
                               .AllowAnyMethod() 
                               .AllowAnyHeader(); 
                    });
            });

            services.AddSignalR(options =>
            {
                options.EnableDetailedErrors = true;
            }).AddJsonProtocol(options =>
            {
                options.PayloadSerializerOptions.Converters.Add(new JsonStringEnumConverter());
            });
            services.AddAutoMapper(AppDomain.CurrentDomain.GetAssemblies());
            services.Configure<DbSettings>(Configuration.GetSection("DbSetting"));
            var dbSettings = new DbSettings();
            Configuration.Bind("DbSetting", dbSettings);
            services.AddFreesql(dbSettings);
            services.AddAutoMapper(AppDomain.CurrentDomain.GetAssemblies());
            services.AddHttpClient();
            services.AddTransient<JwtBearerHandler, JwtTokoenAuthorizeHandler>();
            services.AddTransient<IValidateCodeOperate, ValidateCodeOperate>();

            services.AddSingleton(sp =>
                new MemoryChannel<TransportMessage>(new BoundedChannelOptions(int.Parse(Configuration.GetSection("qu:quMaxCount").Value ?? "10000000")) { FullMode = BoundedChannelFullMode.DropOldest })
            );
            services.AddSingleton(typeof(DataSourceManager));
            services.AddSingleton<IEventPublisher, EventPublisher>()
                .AddSingleton<ISubscriber, Subscriber>()
                .AddTransient<IConsumer<TransportMessage>, TagDataConsumerData>();

            services.AddWS(int.TryParse( Configuration["WSSetting:Port"], out int port)? port: 5000, setting.SecretKey);
        }
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env) 
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }
            app.UseCors("AllowAll");
            app.UseRouting();
            app.UseMiddleware<ErrorHandlingMiddleware>();
            app.UseAuthentication();
            app.UseAuthorization();
            app.UseStaticFiles(new StaticFileOptions
            {
                FileProvider = new PhysicalFileProvider( Path.Combine(env.ContentRootPath, "pic")),  RequestPath = "/pic"
            }); 
            app.UseSwagger();
            app.UseSwaggerUI(c =>
            {
                c.SwaggerEndpoint("v1/swagger.json", "API");
            });
            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
                endpoints.MapFallbackToFile("index.html");
            });
            app.RunConsumer(Configuration);
        }
    }
}
