﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Reflection;
using System.Text;
using OfficeOpenXml;
using System.Threading.Tasks;
using LicenseContext = OfficeOpenXml.LicenseContext;

namespace Ys.Scada.Core
{
    /// <summary>
    /// 表格列操作
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class ExcelCellOption<T>
    {
        /// <summary>
        /// 对应excel中的header字段
        /// </summary>
        public string ExcelField { get; set; }
        /// <summary>
        /// 对应字段的属性(实际上包含PropName)
        /// </summary>
        public PropertyInfo Prop { get; set; }
        /// <summary>
        /// 就是一个看起来比较方便的标识
        /// </summary>
        public string PropName { get; set; }
        /// <summary>
        /// 转换 表格 数据的方法
        /// </summary>
        public Func<string, object> Action { get; set; }
    }
    /// <summary>
    /// 成员信息
    /// </summary>
    public class ExcelMemberInfo
    {
        /// <summary>
        /// 成员信息
        /// </summary>
        public MemberInfo MemberInfo { get; set; }
        /// <summary>
        /// 属性信息
        /// </summary>
        public PropertyInfo PropertyInfo { get; set; }
        /// <summary>
        /// 列名
        /// </summary>
        public string ColumnName { get; set; }
        /// <summary>
        /// 排序
        /// </summary>
        public int Order { get; set; }
    }
    /// <summary>
    /// 特性
    /// </summary>
    [AttributeUsage(AttributeTargets.All)]
    public class ExportAttribute : Attribute
    {
        /// <summary>
        /// 列名
        /// </summary>
        public string ColumnName { get; set; }
        /// <summary>
        /// 是否忽略此属性
        /// </summary>
        public bool Ignore { get; set; }
        /// <summary>
        /// 列的顺序
        /// </summary>
        public int ColumnOrder { get; set; }
    }
    /// <summary>
    /// 表头
    /// </summary>
    public class ExcelHeader
    {
        /// <summary>
        /// 列头名称
        /// </summary>
        public string ColumnName { get; set; }
        /// <summary>
        /// 属性名称
        /// </summary>
        public string PropertyName { get; set; }
        /// <summary>
        /// 列的顺序
        /// </summary>
        public int Order { get; set; }
    }
    /// <summary>
    /// 文件信息
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class ExcelInfo<T>
    {
        public ExcelInfo(List<T> list)
        {
            IncludeMembers = new List<MemberInfo>();
            Headers = new List<ExcelHeader>();
            Init(list);
        }
        /// <summary>
        /// 需要输出的实体对象的成员集合
        /// </summary>
        public List<MemberInfo> IncludeMembers { get; private set; }
        /// <summary>
        /// 表头列的信息集合
        /// </summary>
        public List<ExcelHeader> Headers { get; private set; }
        /// <summary>
        /// 初始化表头和成员集合信息，调用此方法可以过滤掉(不导出)CdExportAttribute中忽略的属性
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="list"></param>
        private void Init(List<T> list)
        {
            var props = typeof(T).GetProperties(BindingFlags.Instance | BindingFlags.Public);
            IncludeMembers = new List<MemberInfo>();
            Headers = new List<ExcelHeader>();
            var epcMemberInfoList = new List<ExcelMemberInfo>();
            foreach (var pi in props)
            {
                var info = new ExcelMemberInfo
                {
                    MemberInfo = pi,
                    PropertyInfo = pi,
                    Order = 0,
                    ColumnName = pi.Name
                };
                var attrs = pi.GetCustomAttributes(typeof(ExportAttribute), true);
                if (attrs.Length <= 0)
                {
                    epcMemberInfoList.Add(info);
                    continue;
                }
                var attr = (ExportAttribute)attrs.FirstOrDefault();
                if (attr == null)
                {
                    epcMemberInfoList.Add(info);
                    continue;
                }
                if (attr.Ignore)
                {
                    continue;
                }
                if (!string.IsNullOrEmpty(attr.ColumnName))
                {
                    info.ColumnName = attr.ColumnName;
                }
                info.Order = attr.ColumnOrder;
                epcMemberInfoList.Add(info);
            }
            IncludeMembers = epcMemberInfoList.OrderBy(x => x.Order).Select(x => x.MemberInfo).ToList();
            var headers = epcMemberInfoList
                .Select(m => new ExcelHeader
                {
                    Order = m.Order,
                    PropertyName = m.PropertyInfo.Name,
                    ColumnName = m.ColumnName
                }).OrderBy(x => x.Order)
                .ToList();
            Headers.AddRange(headers);
        }
    }
    public static class ExcelOperate
    {
        /// <summary>
        /// 导出列表到字节
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="list">数据</param>
        /// <param name="sheetName">表名</param>
        /// <returns></returns>
        public static byte[] ExportToExcelByte<T>(this List<T> list, string sheetName) where T : class
        {
            return ExportToExcelByStream(list, sheetName).ToArray();
        }
        /// <summary>
        /// 导出为流
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="list">数据</param>
        /// <param name="sheetName">表名</param>
        /// <returns></returns>
        public static MemoryStream ExportToExcelByStream<T>(this List<T> list, string sheetName)
        {
            ExcelPackage.LicenseContext = LicenseContext.NonCommercial;
            using var pck = new ExcelPackage();
            //创建工作表
            var ws = pck.Workbook.Worksheets.Add(sheetName);
            var epcExcelInfo = new ExcelInfo<T>(list);
            for (var i = 0; i < epcExcelInfo.Headers.Count(); i++)
            {
                var header = epcExcelInfo.Headers[i];
                ws.Cells[1, i + 1].Value = header.ColumnName;
            }
            //填充数据
            if (list.Count > 0)
            {
                ws.Cells["A2"].LoadFromCollection(list
                                                  , false
                                                  , OfficeOpenXml.Table.TableStyles.None
                                                  , BindingFlags.Instance | BindingFlags.Public
                                                  , epcExcelInfo.IncludeMembers.ToArray());
            }
            ws.Cells.AutoFitColumns();
            //格式化表头
            using (var rng = ws.Cells["A1:BZ1"])
            {
                rng.Style.Font.Bold = true;
            }
            var stream = new MemoryStream();
            pck.SaveAs(stream);
            stream.Position = 0;
            return stream;
        }
        /// <summary>
        /// 将文件导出为实体
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="excelStream">文件流</param>
        /// <param name="sheetIndex">表索引</param>
        /// <param name="options">导出属性</param>
        /// <returns></returns>
        public static ICollection<T> ExcelToEntity<T>(this Stream excelStream, int sheetIndex = 0, ICollection<ExcelCellOption<T>> options = null)
        {
            if (options == null)
                options = new List<ExcelCellOption<T>>().GetExcelCellOptions();
            ExcelPackage.LicenseContext = LicenseContext.NonCommercial;
            using ExcelPackage pack = new(excelStream);
            var sheet = pack.Workbook.Worksheets[sheetIndex];
            int rowCount = sheet.Dimension.Rows, colCount = sheet.Dimension.Columns;
            // 获取对应设置的 表头 以及其 column下标
            var header = sheet.Cells[1, 1, 1, colCount]
                .Where(item => options.Any(opt => opt.ExcelField == item.Value?.ToString().Trim()))
                .ToDictionary(item => item.Value?.ToString().Trim(), item => item.End.Column);
            List<T> data = new();
            // 将excel 的数据转换为 对应实体
            for (int r = 2; r <= rowCount; r++)
            {
                // 将单行数据转换为 表头:数据 的键值对
                var rowData = sheet.Cells[r, 1, r, colCount]
                    .Where(item => header.Any(title => title.Value == item.End.Column)).Select(item => new KeyValuePair<string, string>(header.First(title => title.Value == item.End.Column).Key, item.Value?.ToString().Trim()))
                    .ToDictionary(item => item.Key, item => item.Value);
                var obj = Activator.CreateInstance(typeof(T));
                // 根据对应传入的设置 为obj赋值
                foreach (var option in options)
                {
                    if (!string.IsNullOrEmpty(option.ExcelField))
                    {
                        var value = rowData.ContainsKey(option.ExcelField) ? rowData[option.ExcelField] : string.Empty;
                        if (!string.IsNullOrEmpty(value))
                            option.Prop.SetValue(obj, option.Action == null ? value : option.Action(value));
                    }
                    // 可以用来初始化与表格无关的字段 如 创建时间 Guid主键 之类的东西
                    else
                        option.Prop.SetValue(obj, option.Action == null ? null : option.Action(string.Empty));
                }
                data.Add((T)obj);
            }
            return data;
        }
        /// <summary>
        /// 获取导入列配置信息
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="options">操作</param>
        /// <returns></returns>
        public static ICollection<ExcelCellOption<T>> GetExcelCellOptions<T>(this ICollection<ExcelCellOption<T>> options)
        {
            var model = Activator.CreateInstance(typeof(T));
            var pis = model.GetType().GetProperties();
            foreach (var pi in pis)
            {
                var attrs = pi.GetCustomAttributes(typeof(ExportAttribute), true);
                var attr = (ExportAttribute)attrs.FirstOrDefault();
                options.Add(new ExcelCellOption<T>
                {
                    ExcelField = attr.ColumnName,
                    Prop = pi,
                    PropName = pi.Name,
                    Action = (a) => { return Convert.ChangeType(a, pi.PropertyType); }
                });
            }
            return options;
        }
    }
}
