﻿using Confluent.Kafka;
using Confluent.Kafka.Admin;
using Microsoft.Extensions.Options;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Ys.Scada.Core;
using Ys.Scada.Core.MQ;

namespace Ys.Scada.Kafka
{
    public class KafkaConsumerClient : IConsumerClient
    {
        private static readonly SemaphoreSlim ConnectionLock = new(1, 1);
        private readonly string _groupId;
        private readonly KafkaOptions _kafkaOptions;
        private IConsumer<string, byte[]>? _consumerClient;
        public KafkaConsumerClient(string groupId, IOptions<KafkaOptions> options)
        {
            _groupId = groupId;
            _kafkaOptions = options.Value ?? throw new ArgumentNullException(nameof(options));
        }
        public Func<TransportMessage, object?, Task>? OnMessageCallback { get; set; }
        public Action<LogMessageEventArgs>? OnLogCallback { get; set; }
        public BrokerAddress BrokerAddress => new("Kafka", _kafkaOptions.Servers);
        public void Subscribe(IEnumerable<string> topics)
        {
            if (topics == null) throw new ArgumentNullException(nameof(topics));
            Connect();
            _consumerClient!.Subscribe(topics);
        }
        public void Listening(TimeSpan timeout, CancellationToken cancellationToken)
        {
            Connect();
            while (!cancellationToken.IsCancellationRequested)
            {
                ConsumeResult<string, byte[]> consumerResult;
                try
                {
                    consumerResult = _consumerClient!.Consume(timeout);
                    if (consumerResult == null) continue;
                }
                catch (ConsumeException e) when (_kafkaOptions.RetriableErrorCodes.Contains(e.Error.Code))
                {
                    continue;
                }
                if (consumerResult.IsPartitionEOF || consumerResult.Message.Value == null) continue;
                var headers = new Dictionary<string, string?>(consumerResult.Message.Headers.Count);
                foreach (var header in consumerResult.Message.Headers)
                {
                    var val = header.GetValueBytes();
                    headers.Add(header.Key, val != null ? Encoding.UTF8.GetString(val) : null);
                }
                headers.Add(Core.MQ. Headers.Group, _groupId);
                var message = new TransportMessage(headers, consumerResult.Message.Value);
                OnMessageCallback!(message, consumerResult).GetAwaiter().GetResult();
            }
        }
        public void Connect()
        {
            if (_consumerClient != null) return;
            ConnectionLock.Wait();
            try
            {
                if (_consumerClient == null)
                {
                    var config = new ConsumerConfig(new Dictionary<string, string>(_kafkaOptions.MainConfig));
                    config.BootstrapServers ??= _kafkaOptions.Servers;
                    config.GroupId ??= _groupId;
                    config.AutoOffsetReset ??= AutoOffsetReset.Earliest;
                    config.AllowAutoCreateTopics ??= true;
                    config.EnableAutoCommit ??= false;
                    config.LogConnectionClose ??= false;

                    _consumerClient = BuildConsumer(config);
                }
            }
            finally
            {
                ConnectionLock.Release();
            }
        }
        protected virtual IConsumer<string, byte[]> BuildConsumer(ConsumerConfig config)
        {
            return new ConsumerBuilder<string, byte[]>(config)
                .SetErrorHandler(ConsumerClient_OnConsumeError)
                .Build();
        }
        private void ConsumerClient_OnConsumeError(IConsumer<string, byte[]> consumer, Error e)
        {
        }
        public void Dispose()
        {
            _consumerClient?.Close();
            _consumerClient?.Dispose();
        }
    }
}
