﻿using Microsoft.Extensions.Options;
using MQTTnet;
using MQTTnet.Client;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Ys.Scada.Core;
using Ys.Scada.Core.MQ;

namespace Ys.Scada.MQTT
{
    public class MQTTConsumerClient : IConsumerClient
    {
        private static readonly SemaphoreSlim ConnectionLock = new(1, 1);
        private readonly MQTTOptions _mqttOptions;
        public Func<TransportMessage, object?, Task>? OnMessageCallback { get; set; }
        private MqttClient _consumerClient;
        public BrokerAddress BrokerAddress => new("RabbitMQ", $"{_mqttOptions.Server}:{_mqttOptions.Port}");
        private readonly MemoryChannel<TransportMessage> _memoryChannel;
        private int _id;
        private readonly Producer<TransportMessage> _producer;
        public MQTTConsumerClient(IOptions<MQTTOptions> options, MemoryChannel<TransportMessage> memoryChannel, int id)
        {
            _mqttOptions = options.Value;
            _memoryChannel = memoryChannel;
            _producer = new Producer<TransportMessage>(_memoryChannel.Channel.Writer);
            _id = id;
        }
        public void Connect()
        {
            if (_consumerClient != null) return;
            ConnectionLock.Wait();
            try
            {
                if (_consumerClient == null)
                {
                    var mqttFactory = new MqttFactory();
                    _consumerClient = mqttFactory.CreateMqttClient() as MqttClient;
                    var mqttClientOptions = new MqttClientOptionsBuilder().WithTcpServer(_mqttOptions.Server, _mqttOptions.Port).Build();
                    _consumerClient!.ConnectAsync(mqttClientOptions, CancellationToken.None).GetAwaiter().GetResult();
                }
            }
            finally
            {
                ConnectionLock.Release();
            }
        }
        public void Listening(TimeSpan timeout, CancellationToken cancellationToken)
        {
            Connect();
            _consumerClient.ApplicationMessageReceivedAsync += _consumerClient_ApplicationMessageReceivedAsync;
            while (true)
            {
                cancellationToken.ThrowIfCancellationRequested();
                cancellationToken.WaitHandle.WaitOne(timeout);
            }
        }
        private async Task _consumerClient_ApplicationMessageReceivedAsync(MqttApplicationMessageReceivedEventArgs arg)
        {
            var headers = new Dictionary<string, string?>();
            headers.Add("topic", arg.ApplicationMessage.Topic);
            headers.Add("clientid", arg.ClientId);
            var message = new TransportMessage(headers, arg.ApplicationMessage.Payload);

            await _producer.PublishAsync(message);
        }
        public async void Subscribe(IEnumerable<string> topics)
        {
            if (topics == null) throw new ArgumentNullException(nameof(topics));
            Connect();
            _consumerClient.SubscribeAsync(topics.First(), _mqttOptions.WillQualityOfServiceLevel).GetAwaiter().GetResult();
        }
        public void Dispose()
        {
            _consumerClient?.Dispose();
        }
    }
}
