﻿using Google.Protobuf.WellKnownTypes;
using Newtonsoft.Json;
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.Linq;
using System.Net.Http.Headers;
using System.Text;
using System.Threading;
using System.Threading.Channels;
using System.Threading.Tasks;
using Ys.Scada.Core;
using Ys.Scada.Core.Common;
using Ys.Scada.Core.EventModels;
using Ys.Scada.Core.MQ;
using Ys.Scada.RabbitMQ;
using Ys.Scada.Repository.Domain;

namespace Ys.Scada.Manager
{
    public class ClientManager
    {
        public IConsumerClient Consumer { get; set; }
        public CancellationTokenSource CTS { get; set; }
        public ClientManager() { }
    }
    public class Tag
    {
        public Tag(string name, string type, bool israndom)
        {
            Name = name;
            Type = type;
            IsRandom = israndom;
        }
        public string Name { get; set; }
        public string Type { get; set; }
        public bool IsRandom { get; set; }
        public string Value { get; set; }
    }
    public class DataSourceManager : IDisposable
    {
        private readonly MemoryChannel<TransportMessage> _memoryChannel;
        private readonly Producer<TransportMessage> _producer;
        private DedicatedThreadTaskScheduler _scheduler;
        private readonly IFreeSql _freeSql;
        List<Tag> Tags { get; }

        private readonly ConcurrentDictionary<string, ClientManager> _clients;
        public DataSourceManager(MemoryChannel<TransportMessage> memoryChannel, IFreeSql freeSql)
        {
            _clients = new ConcurrentDictionary<string, ClientManager>();
            _memoryChannel = memoryChannel;
            _scheduler = new DedicatedThreadTaskScheduler(100);
            _freeSql = freeSql;

            Tags = new List<Tag>();
            TempData();
        }
        public void AddOrUpdate(string key, ClientManager obj)
        {
            _clients.AddOrUpdate(key, obj, (n, t) => obj);
        }
        public void Remove(string id)
        {
            if (_clients.ContainsKey(id))
                _clients.Remove(id, out ClientManager client);
        }
        public void Dispose()
        {
            foreach (var client in _clients.Values) 
                client.CTS.Cancel();
            _clients.Clear();
        }
        public Task Cancel(string key)
        {
            if (_clients.TryGetValue(key, out ClientManager client))
                client.CTS.Cancel();
            Remove(key);
            return Task.CompletedTask;
        }
        public  Task Create(DataSource dataSource)
        {
            switch (dataSource.Type.ToLower())
            {
                case "rabbitmq":
                    {
                        var cts = new CancellationTokenSource();
                        _  =  Task.Factory.StartNew(() =>
                        {
                            try
                            {
                                var option = JsonConvert.DeserializeObject<RabbitMQOptions>(dataSource.Data);
                                using (var client = new RabbitMQConsumerClient(option, _memoryChannel, dataSource.Id))
                                {
                                    client.OnStopCallback = StopCallback;
                                    AddOrUpdate(dataSource.Id.ToString(), new ClientManager
                                    {
                                        Consumer = client,
                                        CTS = cts
                                    });
                                    client.Subscribe(new List<string>() { option.Topic });
                                    client.Listening(TimeSpan.FromSeconds(60), cts.Token);
                                }
                            }
                            catch (Exception ex)
                            {
                                if (_clients.ContainsKey(dataSource.Id.ToString()))
                                    _clients.Remove(dataSource.Id.ToString(), out ClientManager client);
                            }
                        }, cts.Token, TaskCreationOptions.LongRunning, TaskScheduler.Default);
                    }
                    break;
                default:
                    break;
            }
            return Task.CompletedTask;
        }
        public void TempData()
        {
            Tags.Add(new Tag("LAC5_2143_AUTO", "Boolean", false));
            Tags.Add(new Tag("LAC5_2143_BELT", "Boolean", false));
            Tags.Add(new Tag("LAC5_2143_BUS", "Boolean", false));
            Tags.Add(new Tag("LAC5_2143_CLEAR", "Boolean", false));
            Tags.Add(new Tag("LAC5_2143_DL", "Boolean", false));
            Tags.Add(new Tag("LAC5_2143_Flow_SET", "Float", true));
            Tags.Add(new Tag("LAC5_2143_FlowReal", "Float", true));
            Tags.Add(new Tag("LAC5_2143_FlowTotal", "Float", true));
            Tags.Add(new Tag("LAC5_2143_Speed", "Float", true));
            Tags.Add(new Tag("LAC5_2143_STATE", "Float", true));
            Tags.Add(new Tag("LAC5_2146_STATE", "Float", true));
            Tags.Add(new Tag("LAC5_2146_STATEA", "Float", true));
            Tags.Add(new Tag("LAC5_2151_ROAD_S", "Float", true));
            Tags.Add(new Tag("LAC5_3374_FlowReal", "Float", true));
            Tags.Add(new Tag("LAC5_DP2140_STATE1", "Float", false));
            Tags.Add(new Tag("LAC5_DP2140_STATE1_LRUN", "Boolean", true));
            Tags.Add(new Tag("LAC5_DP2140_STATE2", "Float", false));
            Tags.Add(new Tag("LAC5_SJ2147_AddFeed_CV", "Float", true));
            Tags.Add(new Tag("LAC5_SJ2147_AddFeed_CV_H", "Float", true));
            Tags.Add(new Tag("LAC5_SJ2147_AddFeed_CV_L", "Float", true));
            Tags.Add(new Tag("LAC5_SJ2147_AddFeed_CV_M", "Float", true));
            Tags.Add(new Tag("LAC5_SJ2147_AddFeed_D", "Float", true));
            Tags.Add(new Tag("LAP5_WQ2116_HotWind_UL", "Float", true));
            Tags.Add(new Tag("LAP5_WQ2116_HotWind1_SET", "Float", true));
            Tags.Add(new Tag("LAP5_WQ2116_HotWind_LL", "Float", true));
            Tags.Add(new Tag("LAP5_WQ2116_HotWind_MAN", "Float", true));
            Tags.Add(new Tag("LAP5_WQ2116_HotWind_P", "Float", true));
            Tags.Add(new Tag("LAP5_WQ2126_OutMois_P_H", "Float", true));
            Tags.Add(new Tag("LAP5_WQ2126_OutMois_P_L", "Float", true));
            Tags.Add(new Tag("LAP5_WQ2126_OutMois_PV", "Float", true));
            Tags.Add(new Tag("LAP5_YT602_2A_MIXJHKSSJ", "String", false));
            Tags.Add(new Tag("LAP5_YT602_2A_MIXSTUFFBRAND", "String", false));
        }
        public void Write(string name, string value)
        {
            var tag = Tags.FirstOrDefault(o => o.Name == name);
            if (tag == null)
            {
                throw new CustomException("参数错误");
            }
            tag.Value = value;
        }
        public List<Tag> GetTags() { return Tags; }
        public List<Tag> GetTagsData()
        {
            foreach (var tag in Tags)
            {
                if (tag.IsRandom)
                {
                    if (tag.Type == "String")
                    {
                        tag.Value = Guid.NewGuid().ToString("N");
                    }
                    else if (tag.Type == "Float")
                    {
                        tag.Value = new Random().Next(10, 1000).ToString();
                    }
                    else if (tag.Type == "Boolean")
                    {
                        tag.Value = (bool.TryParse(tag.Value, out bool v) ? !v : false).ToString();
                    }
                }
            }
            return Tags;
        }
        public async Task StopCallback(int id)
        {
            await _freeSql.Update<DataSource>(id)
                .Set(o => o.Status, 0)
                .ExecuteAffrowsAsync();
        }
        public Task TestConnect(DataSource dataSource)
        {
            if (dataSource.Type.ToLower() == "rabbitmq")
            {
                var option = JsonConvert.DeserializeObject<RabbitMQOptions>(dataSource.Data);
                using (var client = new RabbitMQConsumerClient(option, _memoryChannel, 0))
                {
                    if (!client.TestConnect())
                        throw new CustomException("连接失败");
                }
            }
            return Task.CompletedTask;
        }
    }
}
