﻿using Microsoft.Extensions.Options;
using RabbitMQ.Client.Events;
using RabbitMQ.Client;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Ys.Scada.Core.MQ;
using System.Reflection;
using Ys.Scada.Core.EventModels;
using Ys.Scada.Core;
using System.Threading.Channels;

namespace Ys.Scada.RabbitMQ
{
    public sealed class RabbitMQConsumerClient : IConsumerClient
    {
        private int _id;
        private readonly RabbitMQOptions _rabbitMQOptions;
        private static object _syncLock = new();
        private IModel? _channel;
        private IConnection _connection;
        private readonly MemoryChannel<TransportMessage> _memoryChannel;
        private readonly Producer<TransportMessage> _producer;

        public RabbitMQConsumerClient(RabbitMQOptions options, MemoryChannel<TransportMessage> memoryChannel, int id)
        {
            _rabbitMQOptions = options;
            _memoryChannel = memoryChannel;
            _producer = new Producer<TransportMessage>(_memoryChannel.Channel.Writer);
            _id = id;
        }
        public Func<TransportMessage, object?, Task>? OnMessageCallback { get; set; }
        public Func<int, Task>? OnStopCallback { get; set; }
        public BrokerAddress BrokerAddress => new("RabbitMQ", $"{_rabbitMQOptions.HostName}:{_rabbitMQOptions.Port}");
        public void Subscribe(IEnumerable<string> topics)
        {
            if (topics == null) throw new ArgumentNullException(nameof(topics));
            Connect();
            foreach (var topic in topics)
            {
                _channel.QueueBind(_rabbitMQOptions.QueueName, _rabbitMQOptions.ExchangeName, topic);
            }
        }
        public void Listening(TimeSpan timeout, CancellationToken cancellationToken)
        {
            Connect();
            var consumer = new AsyncEventingBasicConsumer(_channel);
            consumer.Received += OnConsumerReceived;
            consumer.Shutdown += OnConsumerShutdown;
            consumer.ConsumerCancelled += OnConsumerConsumerCancelled;


            if (_rabbitMQOptions.BasicQosOptions != null)
                _channel?.BasicQos(0, _rabbitMQOptions.BasicQosOptions.PrefetchCount, _rabbitMQOptions.BasicQosOptions.Global);
            try
            {
                _channel.BasicConsume(_rabbitMQOptions.QueueName, true, consumer);
            }
            catch (TimeoutException ex)
            {
                OnConsumerShutdown(null, new ShutdownEventArgs(ShutdownInitiator.Application, 0, ex.Message + "-->" + nameof(_channel.BasicConsume)));
            }
            while (true)
            {
                cancellationToken.ThrowIfCancellationRequested();
                cancellationToken.WaitHandle.WaitOne(timeout);
            }
        }
        public void Dispose()
        {
            _channel?.Dispose();
        }
        private IConnection CreateConnection(RabbitMQOptions options)
        {

            if (_connection != null && _connection.IsOpen) return _connection;
            var factory = new ConnectionFactory
            {
                UserName = options.UserName,
                Port = options.Port,
                Password = options.Password,
                DispatchConsumersAsync = true,
                ClientProvidedName = Assembly.GetEntryAssembly()?.GetName().Name!.ToLower()
            };
            if (options.HostName.Contains(","))
            {
                options.ConnectionFactoryOptions?.Invoke(factory);
                return factory.CreateConnection(AmqpTcpEndpoint.ParseMultiple(options.HostName));
            }
            factory.HostName = options.HostName;
            options.ConnectionFactoryOptions?.Invoke(factory);

            return factory.CreateConnection();
        }
        public bool TestConnect()
        {
            try
            {
                CreateConnection(_rabbitMQOptions);
            }
            catch
            {
                return false;
            }
            return true;
        }
        public void Connect()
        {
            lock (_syncLock)
            {
                _connection = CreateConnection(_rabbitMQOptions);
                if (_channel == null || _channel.IsClosed)
                {
                    _channel = _connection.CreateModel();
                    _channel.ExchangeDeclare(_rabbitMQOptions.ExchangeName, _rabbitMQOptions.ExchangeType, true);
                    var arguments = new Dictionary<string, object> { };
                    if (_rabbitMQOptions.QueueArguments != null)
                    {
                        arguments.Add("x-message-ttl", _rabbitMQOptions.QueueArguments.MessageTTL);
                        if (!string.IsNullOrEmpty(_rabbitMQOptions.QueueArguments.QueueMode))
                            arguments.Add("x-queue-mode", _rabbitMQOptions.QueueArguments.QueueMode);
                        if (!string.IsNullOrEmpty(_rabbitMQOptions.QueueArguments.QueueType))
                            arguments.Add("x-queue-type", _rabbitMQOptions.QueueArguments.QueueType);
                    }
                    try
                    {
                        _channel.QueueDeclare(_rabbitMQOptions.QueueName, true, false, false, arguments);
                    }
                    catch (TimeoutException ex)
                    {
                        OnConsumerShutdown(null, new ShutdownEventArgs(ShutdownInitiator.Application, 0, ex.Message + "-->" + nameof(_channel.QueueDeclare)));
                    }
                }
            }
        }
        private async Task OnConsumerConsumerCancelled(object? sender, ConsumerEventArgs e)
        {
            await OnStopCallback(_id);
        }
        private async Task OnConsumerReceived(object? sender, BasicDeliverEventArgs e)
        {
            var headers = new Dictionary<string, string?>();
            if (e.BasicProperties.Headers != null)
                foreach (var header in e.BasicProperties.Headers)
                    if (header.Value is byte[] val)
                        headers.Add(header.Key, Encoding.UTF8.GetString(val));
                    else
                        headers.Add(header.Key, header.Value?.ToString());

            var message = new TransportMessage(headers, e.Body.ToArray());
            await _producer.PublishAsync(message);
        }
        private async Task OnConsumerShutdown(object? sender, ShutdownEventArgs e)
        {
            await OnStopCallback(_id);
        }
    }
}
